/**
  SwitchHosts
  Copyright (C) 2009-2009 Takashi Ishikawa.
  http://switchhosts.mozdev.org/

  This source code is released under the GPL license,
  available in the LICENSE file at the root of this installation
  and also online at http://www.gnu.org/licenses/gpl.txt
**/

Components.utils.import("resource://modules/PaidonUtil.jsm");
Components.utils.import("resource://modules/PaidonLogger.jsm");

var EXPORTED_SYMBOLS = ["PaidonDataSource"];

const Cc = Components.classes;
const Ci = Components.interfaces;
var Application = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);

function PaidonDataSource(rdfRoot) {

  this.logger = new PaidonLogger(true);

  this.initLocalDs = function() {
    var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
    var io = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    file = file.get("PrefD", Components.interfaces.nsILocalFile);
    file.appendRelativePath("localstore.rdf");
    io = io.newFileURI(file);
    return io.spec;
  };

    //CX^Xϐ
    this.rdfRoot = rdfRoot;

    this.rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
    this.ds = this.rdfService.GetDataSourceBlocking(this.initLocalDs());
    this.rdfUtil = Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils);
    this.rdfContainer = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);

    //Get or Add Sequence
  try{
    this.rdfContainer.Init(this.ds, this.rdfService.GetResource(this.rdfRoot)); //Get
  }catch(e){
    this.rdfContainer = this.rdfUtil.MakeSeq(this.ds, this.rdfService.GetResource(this.rdfRoot)); //Create
  }

  this.getAllElements = function() {
    try{
      var aOut		= new Array();
      var aElements	= this.rdfContainer.GetElements();
      while(aElements.hasMoreElements()){
        var oRes = aElements.getNext();
          oRes = oRes.QueryInterface(Components.interfaces.nsIRDFResource);

        aOut[aOut.length] = oRes.Value;
      }
      return aOut;
    }catch(e){
      this.logger.error(e);
      throw e;
    }
  };

  this.getElement = function(sUri){
    try{
      return this.rdfService.GetResource(sUri);
    }catch(e){
      this.logger.error(e);
      throw e;
    }
  };

  this.getValue = function(oRes, oProp){
    try{
      var oTrgt = this.ds.GetTarget(oRes, oProp, true);

      if(oTrgt instanceof Components.interfaces.nsIRDFLiteral){
        return oTrgt.Value;
      }

    }catch(e){
      this.logger.error(e);
      throw e;
    }

    return null;
  };

  this.getValues = function(sUri){

    var aOut = new Array();

    //Get an array of elements for sAbout
    try{
      var oRes	= this.getElement(sUri);
      var oTrgts	= this.ds.ArcLabelsOut(oRes, true);
      while(oTrgts.hasMoreElements()){
        var oTrgt = oTrgts.getNext();

        if (oTrgt instanceof Components.interfaces.nsIRDFResource){
          aOut[oTrgt.Value] = this.getValue(oRes, oTrgt);
        }
      }
    }catch(err){throw "(getValues)\n" + err}

    return aOut;
  };

  this.isElementExists = function(sUri){
    try{
      var aElems = this.getAllElements();
      for(var e = 0; e < aElems.length; e++){
        if(aElems[e] == sUri)
          return true;
      }
    }catch(e){
      this.logger.error(e);
      throw e;
    }
  };

  this.addElement = function(sUri){
    try{
      var oRes = this.rdfContainer.AppendElement(this.rdfService.GetResource(sUri));
      return oRes;
    }catch(e){
      this.logger.error(e);
      throw e;
    }
  };

  this.addProperty = function(oRes, oProp, sValue){
    try{
      if(this.ds.hasArcOut(oRes, oProp))
        return;

      this.ds.Assert(oRes, oProp, this.rdfService.GetLiteral(sValue), true);

    }catch(e){
      this.logger.error(e);
      throw e;
    }
  };

  this.removeElement = function(sUri){
    try{
      var oRes = this.rdfService.GetResource(sUri);

      //Remove All Archs
      // Loop for duplicates
      var aArchs		= null;
      var hasArchs	= true;
      while(hasArchs){
        aArchs		 = this.ds.ArcLabelsOut(oRes);
        hasArchs	 = aArchs.hasMoreElements();
        while(aArchs.hasMoreElements()){
          oArch = aArchs.getNext().QueryInterface(Components.interfaces.nsIRDFResource);

          //Remove
          this.ds.Unassert(oRes, oArch, this.ds.GetTarget(oRes, oArch, true));
        }
      }

      //Remove Element
      this.rdfContainer.RemoveElement(oRes, true);
    }catch(e){
      this.logger.error(e);
      throw e;
    }
  };

  this.changeElementUri = function(oRes, sNewUri){
    try{

      //Get All Properties for element
      var aProps = this.getValues(oRes.Value);

      //Remove Element
      this.removeElement(oRes.Value);

      //Create element again with new URI
      var newElem = this.rdfService.GetResource(sNewUri);
      this.rdfContainer.AppendElement(newElem);
      for(sProp in aProps){
        this.ds.Assert(newElem, this.rdfService.GetResource(sProp), this.rdfService.GetLiteral(aProps[sProp]), true);
      }

    }catch(err){throw "(changeElementUri)\n" + err}
  };

  this.changeProperty = function(oRes, oProp, sValue){
    try{

      //Get old value
      var sOld = this.getValue(oRes, oProp);

      //Change
      this.ds.Change(oRes, oProp, this.rdfService.GetLiteral(sOld), this.rdfService.GetLiteral(sValue));

    }catch(err){throw "(changeProperty)\n" + err}
  }

};

