/**
  SwitchHosts
  Copyright (C) 2009-2009 Takashi Ishikawa.
  http://switchhosts.mozdev.org/

  This source code is released under the GPL license,
  available in the LICENSE file at the root of this installation
  and also online at http://www.gnu.org/licenses/gpl.txt
**/

Components.utils.import("resource://modules/PaidonUtil.jsm");
Components.utils.import("resource://modules/PaidonFile.jsm");
Components.utils.import("resource://modules/PaidonDns.jsm");
Components.utils.import("resource://modules/SWConst.jsm");
Components.utils.import("resource://modules/SWFactory.jsm");
Components.utils.import("resource://modules/SWHostsFile.jsm");
Components.utils.import("resource://modules/SWProfile.jsm");
Components.utils.import("resource://modules/SWUtil.jsm");

var EXPORTED_SYMBOLS = ["SWHostsManager"];

const Cc = Components.classes;
const Ci = Components.interfaces;
var Application = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);
var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
var navigator = wm.getMostRecentWindow('navigator:browser');
var browser = wm.getEnumerator('navigator:browser').getNext().getBrowser();
var downloadManager = Cc["@mozilla.org/download-manager;1"].getService(Ci.nsIDownloadManager);

function SWHostsManager() {

  this.logger = SWFactory.getLogger(this);

  this.aHostsFileList = new Array();

  this.datasource = SWFactory.getDataSource();
  this.preference = SWFactory.getPreference();
  this.resource = SWFactory.getResource();
  this.observer = SWFactory.getObserver();

  this.getList = function(){
    return this.aHostsFileList;
  };

  this.reloadList = function(){
    this.aHostsFileList = new Array();
    var aHosts = this.datasource.getAllElements();

    //Add to list
    for(var i = 0; i < aHosts.length; i++){

      if(typeof(aHosts[i]) == 'undefined')
        continue;

      var oRes		= this.datasource.getElement(aHosts[i]);
      var sHostsName	= this.datasource.getValue(oRes, this.datasource.getElement(SWConst.RDF_NODE_ROOT + "#name"));
      var sHostsPath	= this.datasource.getValue(oRes, this.datasource.getElement(SWConst.RDF_NODE_ROOT + "#path"));

      var hostsFile = new SWHostsFile();
      hostsFile.setName(sHostsName);
      hostsFile.setUri(aHosts[i]);
      hostsFile.setPath(sHostsPath);

      this.aHostsFileList.push(hostsFile);

    }

    return this.aHostsFileList;
  };

  this.readFile = function(uri){
    var alist = this.getList();
    for(var i = 0; i < alist.length; i++){
      if(alist[i].getUri() == uri){
        alist[i].read();
        break;
      }
    }
    return alist[i];
  };

  this.getFile = function(uri){
    var alist = this.getList();
    for(var i = 0; i < alist.length; i++){
      if(alist[i].getUri() == uri){
        break;
      }
    }
    return alist[i];
  };

  this.getFileByName = function(name){
    var alist = this.getList();
    for(var i = 0; i < alist.length; i++){
      if(alist[i].getName() == name){
        break;
      }
    }
    return alist[i];
  };

  this.cleanUpList = function(){

    this.reloadList();

    var alist = this.getList();

    //f[^\[XɂāAt@C݂Ȃ̂폜
    for(var i = 0; i < alist.length; i++){
this.logger.debug(alist[i].getUri());
this.logger.debug(alist[i].getPath());
      if(!alist[i].getPath()){
        this.datasource.removeElement(alist[i].getUri());
      }
      var file = new PaidonFile(alist[i].getPath());
      if(alist[i].getPath() != SWProfile.getSaveHostsPath() + alist[i].getFileName() || !file.exists()){
        this.datasource.removeElement(alist[i].getUri());
      }
    }

    //f[^\[XĂ̂ŁAēxĂ
    this.reloadList();
    alist = this.getList();

    var hostsFile = new SWHostsFile();
    file = new PaidonFile(SWProfile.getSaveHostsPath());

    var aFiles = file.getFiles(SWConst.HOSTS_FILE_EXT);

    //t@CāAf[^\[XɂȂ̂쐬
    for(var i = 0; i < aFiles.length; i++){
      var flg = false;
      for(var r = 0; r < alist.length; r++){
        if(alist[r].getFileName() == aFiles[i]){
          flg = true;
        }
      }
      if(!flg){
        hostsFile = new SWHostsFile();
        hostsFile.setName(aFiles[i].substring(0, aFiles[i].length - SWConst.HOSTS_FILE_EXT.length));
        hostsFile.setPath(SWProfile.getSaveHostsPath() + aFiles[i]);
        hostsFile.read();
        this.saveFile(hostsFile, "add");
      }
    }

    this.reloadList();
    alist = this.getList();

    if(alist.length == 0){
      //1LhostsOriginal쐬
      this.createDefault();
      this.reloadList();
    }

    //HostsIԂɂ
    this.changeCurrentHostsFile();

  };

  this.createDefault = function(){
    var oDefaultHostsFile = new SWHostsFile();
    oDefaultHostsFile.setName(this.resource.getText("hosts.default"));
    oDefaultHostsFile.setPath(SWProfile.getSaveHostsPath() + oDefaultHostsFile.getFileName());
    oDefaultHostsFile.setUri(SWConst.RDF_ROOT + "/hosts_default");
    //ftHgHostsFile̍쐬
    var file = new PaidonFile(SWProfile.getOsHostsPath());
    file.copy(SWProfile.getSaveHostsPath(), oDefaultHostsFile.getFileName());
    this.addDs(oDefaultHostsFile);
  };

  this.startAudit = function(){
    var file = new PaidonFile(SWProfile.getOsHostsPath());
    this.iHostsModifiedTime = file.getLastModifiedTime();
    this.iFileSize = file.getFileSize();
    this.timer = Components.classes['@mozilla.org/timer;1'].createInstance(Components.interfaces.nsITimer);
    this.timer.init(this, 1000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
  };

  this.observe = function(aSubject, aTopic, aData){
    if (aTopic != 'timer-callback') return;
    var file = new PaidonFile(SWProfile.getOsHostsPath());

    if(!file.exists() || !file.isReadable()){
      return;
    }

    if(file.getLastModifiedTime() != this.iHostsModifiedTime || file.getFileSize() != this.iFileSize){
      var oHosts = this.getCurrentHostsFile();
      var sHost = this.dns.getCurrentHost();
      var aIp = this.dns.getCurrentIps();
      this.observer.notifyOsHostsChanged(oHosts.getName(), sHost, aIp);
      this.iHostsModifiedTime = file.getLastModifiedTime();
      this.iFileSize = file.getFileSize();
    }

  };

  this.changeCurrentHostsFile = function(){
    var currentHostsUri = "";
    if(this.preference.exists("hosts.current")){
      currentHostsUri = this.preference.getString("hosts.current");
      this.currentHostsFile = this.getFile(currentHostsUri);
      if(!this.currentHostsFile){
        alist = this.getList();
        this.currentHostsFile = alist[0];
        this.preference.setString("hosts.current", this.currentHostsFile.getUri());
      }
    }else{
      alist = this.getList();
      this.currentHostsFile = alist[0];
      this.preference.setString("hosts.current", this.currentHostsFile.getUri());
    }
  };

  this.select = function(name, uri){

    var sBeforeUri = this.getCurrentHostsFile().getUri();

    this.preference.setString("hosts.current", uri);
    this.changeCurrentHostsFile();

    if(!this.refreshDns()){
      //hosts̏㏑ɎŝŁAɖ߂B
      this.preference.setString("hosts.current", sBeforeUri);
      this.changeCurrentHostsFile();
      return;
    }

    //notify
    this.observer.notifySelectHosts(name);

  };

  this.refreshDns = function(){

    var activeDownloads = downloadManager.activeDownloadCount;

    if (activeDownloads > 0 && !SWUtil.confirmByKey("confirm.stop.download")) {
        return false;
    }

    //ꎞ~
    this.timer.cancel();

    //hostsOSt@Cɏ㏑BsAlert
    if(!this.getCurrentHostsFile().apply()){
      this.startAudit();
      return false;
    }

    this.startAudit();

    this.dns.refresh();

    return true;

  };

  this.getCurrentHostsFile = function(){
    return this.currentHostsFile;
  };

  this.removeFile = function(sUri){
    var hosts = this.getFile(sUri);

    if(hosts){
      var file = new PaidonFile(hosts.getPath());
      file.remove();
    }

    this.datasource.removeElement(sUri);

    this.reloadList();

    //notify
    this.observer.notifyRemoveHostsFile(sUri);

};

  this.saveFile = function(oHostsFile, sOperation){

      var beforePath = oHostsFile.getPath();
      oHostsFile.setPath(SWProfile.getSaveHostsPath() + oHostsFile.getFileName());
      oHostsFile.write();

      //OCꍇ͌t@C폜
      if(sOperation == "edit"){
        this.saveChangeNameDs(oHostsFile);
        if(beforePath != oHostsFile.getPath()){
          var file = new PaidonFile(beforePath);
          file.remove();
        }
      }

      if(sOperation == "add" || sOperation == "copy"){
        this.addDs(oHostsFile);
      }

      this.reloadList();

      //notify
      this.observer.notifySaveHostsFile(oHostsFile.getName());

      //I𒆂HostsύXꂽꍇ
      if(this.getCurrentHostsFile() && this.getCurrentHostsFile().getUri() == oHostsFile.getUri()){
        this.select(oHostsFile.getName(), oHostsFile.getUri());
      }

  };

  //ҏWɂ薼Oςꍇ
  this.saveChangeNameDs = function(oHostsFile){
      var oElement = this.datasource.getElement(oHostsFile.getUri());
      this.datasource.changeProperty(oElement, this.datasource.getElement(SWConst.RDF_NODE_ROOT + "#name"), oHostsFile.getName());
      this.datasource.changeProperty(oElement, this.datasource.getElement(SWConst.RDF_NODE_ROOT + "#path"), oHostsFile.getPath());
  };

  this.addDs = function(oHostsFile){

      //Element쐬
      oHostsFile.setUri(this.getUniqueUri());

      var oElement = this.datasource.getElement(oHostsFile.getUri());

      //Element쐬
      this.datasource.addElement(oHostsFile.getUri());

      //OZbg
      this.datasource.addProperty(oElement, this.datasource.getElement(SWConst.RDF_NODE_ROOT + "#name"), oHostsFile.getName());

      //t@CpXZbg
      this.datasource.addProperty(oElement, this.datasource.getElement(SWConst.RDF_NODE_ROOT + "#path"), oHostsFile.getPath());

  };

  this.getUniqueUri = function(){
    var sUri = SWConst.RDF_ROOT + "/hosts_" + Math.round((Math.random() * 200000));

    if(this.datasource.isElementExists(sUri))
      return this.getUniqueUri();

    return sUri;
  };

  this.onChangeLocation = function(sHost, aIp, topic){
    var oHosts = this.getCurrentHostsFile();
    //notify
    if(topic  == SWConst.EVENT_LOCATION_CHANGE ){
      this.observer.notifyChangeLocation(oHosts.getName(), sHost, aIp);
    }else if(topic == SWConst.EVENT_REFRESH_DNS){
      this.observer.notifyRefreshDns(oHosts.getName(), sHost, aIp);
    }
  };

  this.addObserver = function(object, eventName){
    this.observer.addObserver(object, eventName, false);
  };

  this.changeSaveHostsPath = function(sBeforePath, sAfterPath){
    if(sBeforePath == sAfterPath){
      return;
    }
    var files = new PaidonFile(sBeforePath);
      var aFiles = files.getFiles(SWConst.HOSTS_FILE_EXT);
      for(var i = 0; i < aFiles.length; i++){
        oAfterfile = new PaidonFile(SWProfile.getDirectoryPath(sAfterPath) + aFiles[i]);
        oBeforeFile = new PaidonFile(SWProfile.getDirectoryPath(sBeforePath) + aFiles[i]);
        if(!oAfterfile.exists()){
          oBeforeFile.copy(SWProfile.getDirectoryPath(sAfterPath));
        }else{
          if(SWUtil.confirmByKey("hosts.move.confirm", [aFiles[i], SWProfile.getDirectoryPath(sAfterPath)])){
            oAfterfile.remove();
            oBeforeFile.copy(SWProfile.getDirectoryPath(sAfterPath));
          }
        }
        //pXςB
        var oHosts = this.getFileByName(aFiles[i].substring(0, aFiles[i].length - SWConst.HOSTS_FILE_EXT.length));
        oHosts.setName(aFiles[i].substring(0, aFiles[i].length - SWConst.HOSTS_FILE_EXT.length));
        oHosts.setPath(SWProfile.getDirectoryPath(sAfterPath) + aFiles[i]);
        this.saveChangeNameDs(oHosts);
      }
      this.cleanUpList();
      this.select(this.getCurrentHostsFile().getName(), this.getCurrentHostsFile().getUri())
      SWUtil.alertInfoByKey("hosts.move.success", [sAfterPath]);
  };

  this.destroy = function(){
    this.timer.cancel();
    this.dns.destroy();
  };

  this.uninstall = function(){
    //preference폜
    this.preference.deleteBranch();
  };

//X^[g

  //OS Hostst@Č`FbN
  if(SWUtil.checkPermission(SWProfile.getOsHostsPath())){

    //cleanup
    this.cleanUpList();

    //Hosts[h
    this.getCurrentHostsFile().apply();

    //Hosts̊Ď
    this.startAudit();

    //DnsݒBonChangeLocationɌĂ΂Ă܂̂ŁAԉɈړ
    this.dns = SWFactory.getDns(this);

    this.isAddonStarted = true;

  }else{
    SWUtil.alertErrorByKey("error.stop.addon");
    this.isAddonStarted = false;
  }

};
